<?php
/**
 * @copyright   Copyright (C) Chatbro. All rights reserved.
 * @license     GNU General Public License version 2 or later.
 */

defined('_JEXEC') or die();
defined('CHATBROENGINE') || die();

use ChatbroChatbroappCoreCbro_backendable as CbroBackendable;
use ChatbroChatbroappUtilsCbro_utils as CbroUtils;
use ChatbroChatbroappExceptionsCBro_store_not_found as CBroStoreNotFound;

class ChatbroChatbroappCommonCbro_common extends CbroBackendable
{
	private static $tableName = '#__chatbro';

	protected function __construct($backend)
	{
		parent::__construct($backend);
	}

	public static function get_store_id()
	{
		$store_id = self::get_backend()->store_id;

		if ($store_id === null || $store_id === '') {
			throw new CBroStoreNotFound('Undefined');
		}

		return (int) self::getDb()->escape($store_id);
	}

	public static function remote_post($uri, $args)
	{
		$curl_options = [];

		if (isset($args['headers'])) {
			$newHeaders = array_map(
				function ($key, $value) {
					return $key . ': ' . $value;
				},
				array_keys($args['headers']),
				$args['headers']
			);

			// Обновление исходного массива
			$args['headers'] = $newHeaders;

			$curl_options[CURLOPT_HTTPHEADER] = $args['headers'];
		}

		if (isset($args['body'])) {
			$curl_options[CURLOPT_POST] = true;
			$curl_options[CURLOPT_POSTFIELDS] = $args['body'];
		}

		$curl_options[CURLOPT_URL] = $uri;
		$curl_options[CURLOPT_RETURNTRANSFER] = true;
		$curl_options[CURLOPT_SSL_VERIFYHOST] = false;
		$curl_options[CURLOPT_SSL_VERIFYPEER] = false;

		$ch = curl_init();
		curl_setopt_array($ch, $curl_options);
		$result = curl_exec($ch);
		$info = curl_getinfo($ch);
		curl_close($ch);

		return [
			'result' => $result,
			'info' => $info,
		];
	}

	public static function remote_retrieve_response_code($response)
	{
		return isset($response['info']['http_code'])
			? (int) $response['info']['http_code']
			: 500;
	}

	public static function remote_retrieve_body($response)
	{
		return isset($response['result']) ? $response['result'] : null;
	}

	public static function is_error($response)
	{
		return (int) $response['info']['http_code'] >= 404;
	}

	public static function delete_access_token()
	{
		self::delete_option('chatbro_access_token');
	}

	public static function delete_refresh_token()
	{
		self::delete_option('chatbro_refresh_token');
	}

	public static function set_access_token($access_token)
	{
		return self::update_option('chatbro_access_token', $access_token);
	}

	public static function set_refresh_token($refresh_token)
	{
		return self::update_option('chatbro_refresh_token', $refresh_token);
	}

	public static function get_access_token()
	{
		return self::get_option('chatbro_access_token');
	}

	public static function get_refresh_token()
	{
		return self::get_option('chatbro_refresh_token');
	}

	private static function getDb()
	{
		return self::get_backend()->getDb();
	}

	public static function get_store($store_id)
	{
		$selectSql = self::getDb()
			->getQuery(true)
			->select('*')
			->from(self::getDb()->quoteName(self::$tableName))
			->where(
				self::getDb()->quoteName('store_id') .
					' = ' .
					self::getDb()->quote($store_id)
			);

		return self::getDb()
			->setQuery($selectSql)
			->loadAssoc();
	}

	public static function reg_store()
	{
		$store_id = self::get_store_id();
		$databaseData = null;

		try {
			$selectSql = self::getDb()
				->getQuery(true)
				->select('*')
				->from(self::getDb()->quoteName(self::$tableName))
				->where(
					self::getDb()->quoteName('store_id') .
						' = ' .
						self::getDb()->quote($store_id)
				);

			$databaseData = self::getDb()
				->setQuery($selectSql)
				->loadAssoc();
		} catch (Exception $e) {
			throw new CBroStoreNotFound(self::get_store_id());
		}

		if (!empty($databaseData)) {
			return;
		}

		try {
			$insertSql = self::getDb()
				->getQuery(true)
				->insert(self::getDb()->quoteName(self::$tableName))
				->columns([
					self::getDb()->quoteName('chatbro_chat_guid'),
					self::getDb()->quoteName('chatbro_chat_secret'),
					self::getDb()->quoteName('store_id'),
				])
				->values(
					self::getDb()->quote(CbroUtils::gen_guid()) .
						', ' .
						self::getDb()->quote(CbroUtils::gen_secret()) .
						', ' .
						self::getDb()->quote($store_id)
				);

			self::getDb()->setQuery($insertSql);
			self::getDb()->execute();
		} catch (Exception $e) {
			throw new CBroStoreNotFound($store_id);
		}
	}

	public static function add_option($name, $value)
	{
		return self::update_option($name, $value);
	}

	public static function update_option($name, $value)
	{
		$store_id = self::get_store_id();
		$updateSql = null;

		try {
			$updateSql = self::getDb()
				->getQuery(true)
				->update(self::getDb()->quoteName(self::$tableName))
				->set(
					self::getDb()->quoteName($name) .
						' = ' .
						self::getDb()->quote($value)
				)
				->set(self::getDb()->quoteName('updated_at') . ' = NOW()')
				->where(
					self::getDb()->quoteName('store_id') .
						' = ' .
						self::getDb()->quote($store_id)
				);

			self::getDb()->setQuery($updateSql);
			self::getDb()->execute();
		} catch (Exception $e) {
			// throw new CBroStoreNotFound($updateSql);
			throw new CBroStoreNotFound(self::get_store_id());
		}
	}

	public static function get_option($name, $defaultValue = null)
	{
		$store_id = self::get_store_id();
		$store = self::get_store($store_id);

		if (empty($store)) {
			throw new CBroStoreNotFound(self::get_store_id());
		}

		if (!isset($store[$name]) || is_null($store[$name])) {
			return $defaultValue;
		}

		return $store[$name];
	}

	public static function delete_option($name)
	{
		$store_id = self::get_store_id();
		$store = self::get_store($store_id);

		if (empty($store)) {
			throw new CBroStoreNotFound(self::get_store_id());
		}

		if (isset($store[$name])) {
			return self::update_option($name, 'NULL');
		}

		return false;
	}

	public static function delete_store()
	{
	}

	public static function generated_scripts()
	{
	}
}
